#include "mainwidow.h"

MainWidow::MainWidow(QWidget *parent, Qt::WindowFlags flags) : QMainWindow(parent, flags) {
  QTabWidget *tabwidgetesterno = new QTabWidget(this);
  tabwidgetesterno->setTabPosition(QTabWidget::West);

  tablistaUPS = new QTabWidget(tabwidgetesterno);
  riassunto = new Riassunto(tabwidgetesterno);
  storico = new Storico(upsList, &db, tabwidgetesterno);
  tabwidgetesterno->addTab(riassunto, tr("Riassunto UPS"));
  tabwidgetesterno->addTab(tablistaUPS, tr("Dettagli UPS"));
  tabwidgetesterno->addTab(storico, tr("Storico UPS"));

  setCentralWidget(tabwidgetesterno);

  QAction *azione;
  QMenu *menu;
  QMenuBar *barramenu = new QMenuBar(this);
  menu = barramenu->addMenu(tr("&File"));
  azione = menu->addAction(tr("&Nuovo UPS ..."));
  connect(azione, SIGNAL(triggered()), SLOT(menu_nuovo()));
  menu->addSeparator();
  azione = menu->addAction(tr("&Modifica UPS ..."));
  connect(azione, SIGNAL(triggered()), SLOT(menu_modifica()));
  menu->addSeparator();
  azione = menu->addAction(tr("&Cancella UPS ..."));
  connect(azione, SIGNAL(triggered()), SLOT(menu_cancella_ups()));
  menu->addSeparator();
  azione = menu->addAction(tr("&Esci"));
  connect(azione, SIGNAL(triggered()), SLOT(menu_esci()));
  menu = barramenu->addMenu(tr("&Impostazioni"));
  azione = menu->addAction(tr("&Programma ..."));
  connect(azione, SIGNAL(triggered()), SLOT(menu_parametri()));
  azione = menu->addAction(tr("Interv&alli dei valori dell'ups ..."));
  connect(azione, SIGNAL(triggered()), SLOT(menu_modifica_intervalli()));
  menu = barramenu->addMenu(tr("&Utility"));
  azione = menu->addAction(tr("&Genera apcupsd.conf ..."));
  menu = barramenu->addMenu(tr("&?"));
  azione = menu->addAction(tr("&Guida rapida"));
  menu->addSeparator();
  azione = menu->addAction(tr("&Informazioni su ..."));

  setMenuBar(barramenu);

  db = QSqlDatabase::addDatabase(QString("QSQLITE"));
  QDir filepath(QDir::homePath());
  filepath.mkdir(QString(".qtapc"));
  filepath.cd(QString(".qtapc"));
  db.setDatabaseName(filepath.filePath(QString("db.sqlite")));
  if(!db.open()) {
    QMessageBox::critical(this, tr("QtAPC errore nel DB"), tr("Non riesco ad aprire il file DB"));
    qApp->exit(1);
  }

  SQL::attivaForeignKey(db);
  SQL::creaTabelle(db);

  QVariant valore;
  valore = SQL::prendiParametro(numeroBarreTxt, db);
  if(valore.isNull()) {
    numerobarre = 4;
    SQL::salvaParametro(numeroBarreTxt, numerobarre, db);
  } else
    numerobarre = valore.toInt();

  valore = SQL::prendiParametro(sfumaturaTxt, db);
  if(valore.isNull()) {
    sfumatura = true;
    SQL::salvaParametro(sfumaturaTxt, sfumatura, db);
  } else
    sfumatura = valore.toBool();

  valore = SQL::prendiParametro(sfumaturaIntervalliTxt, db);
  if(valore.isNull()) {
    sfumaturaIntervalli = true;
    SQL::salvaParametro(sfumaturaIntervalliTxt, sfumaturaIntervalli, db);
  } else
    sfumaturaIntervalli = valore.toBool();

  valore = SQL::prendiParametro(giornidasalvareTxt, db);
  if(valore.isNull()) {
    giornidasalvare = 80;
    SQL::salvaParametro(giornidasalvareTxt, giornidasalvare, db);
  } else
    giornidasalvare = valore.toInt();

  QList<SQL::UpsInfo> listaUps = SQL::prendiUps(db);
  for(int i = 0; i < listaUps.size(); i++) {
    Ups *ups = new Ups(listaUps[i]);
    ups->setDB(&db);
    crea_gui_ups(*ups);
    dynamic_cast<ListaBarre*>(ups->getWidgetBarre())->setNumeroBarre(numerobarre);
    dynamic_cast<ListaBarre*>(ups->getWidgetBarre())->setSfumatura(sfumatura);
    dynamic_cast<ListaBarre*>(ups->getWidgetBarre())->setSfumaturaIntervalli(sfumaturaIntervalli);
    ups->start();
    upsList.push_back(ups);
    storico->aggiornaListaUps();
  }

  resize(600, 450);
}

const QString MainWidow::numeroBarreTxt("Numero barre");
const QString MainWidow::sfumaturaTxt("Sfumatura");
const QString MainWidow::sfumaturaIntervalliTxt("Sfumatura intervalli");
const QString MainWidow::giornidasalvareTxt("Giorni da salvare");

MainWidow::~MainWidow() {
  QList<Ups*>::iterator it;
  it = upsList.begin();
  while(it != upsList.end()) {
    (*it)->stop();
    it++;
  }
  it = upsList.begin();
  while(it != upsList.end()) {
    (*it)->waitStop();
    delete *it;
    it = upsList.erase(it);
  }
  SQL::salvaParametro(giornidasalvareTxt, giornidasalvare, db);
  SQL::salvaParametro(numeroBarreTxt, numerobarre, db);
  SQL::salvaParametro(sfumaturaTxt, sfumatura, db);
  SQL::salvaParametro(sfumaturaIntervalliTxt, sfumaturaIntervalli, db);
  if(giornidasalvare > 0)
    SQL::cancellaUpsStatusVecchi(QDateTime::currentDateTime().addDays(-giornidasalvare), db);
  db.close();
}

void MainWidow::nuova_sfumatura_barre(bool v) {
  sfumatura = v;
  for(int i = 0; i < upsList.size(); i++) {
    upsList[i]->getWidgetBarre()->setSfumatura(v);
    upsList[i]->getWidgetGrafici()->setSfumatura(v);
  }
}

void MainWidow::nuova_sfumatura_intervalli(bool v) {
  sfumaturaIntervalli = v;
  for(int i = 0; i < upsList.size(); i++) {
    upsList[i]->getWidgetBarre()->setSfumaturaIntervalli(v);
    upsList[i]->getWidgetGrafici()->setSfumaturaIntervalli(v);
  }
}

void MainWidow::nuovo_numero_barre(int n) {
  numerobarre = n;
  for(int i = 0; i < upsList.size(); i++) {
    dynamic_cast<ListaBarre*>(upsList[i]->getWidgetBarre())->setNumeroBarre(n);
  }
}

void MainWidow::nuovi_giorni_da_salvare(int g) {
  giornidasalvare = g;
}

void MainWidow::menu_esci() {
  close();
}

void MainWidow::menu_parametri() {
  Parametri *d = new Parametri(numerobarre, sfumatura, sfumaturaIntervalli, giornidasalvare, this);
  connect(d, SIGNAL(nuova_sfumatura_barre(bool)), SLOT(nuova_sfumatura_barre(bool)));
  connect(d, SIGNAL(nuova_sfumatura_intervalli(bool)), SLOT(nuova_sfumatura_intervalli(bool)));
  connect(d, SIGNAL(nuovo_numero_barre(int)), SLOT(nuovo_numero_barre(int)));
  connect(d, SIGNAL(nuovi_giorni_da_salvare(int)), SLOT(nuovi_giorni_da_salvare(int)));
  d->show();
}

void MainWidow::menu_modifica_intervalli() {
  ModificaIntervalliUPS *d = new ModificaIntervalliUPS(upsList, &db, this);
  d->setParametriGlobali(sfumatura, sfumaturaIntervalli, numerobarre);
  d->show();
}

void MainWidow::menu_nuovo() {
  Nuovo *n = new Nuovo(this);
  connect(n, SIGNAL(nuovo_ups(QString, unsigned int, QString)), SLOT(nuovo_ups(QString, unsigned int, QString)));
  n->show();
}

void MainWidow::menu_modifica() {
  Modifica *m = new Modifica(upsList, this);
  connect(m, SIGNAL(updated(QList<Ups*>::Iterator)), SLOT(update_name(QList<Ups*>::Iterator)));
  m->show();
}

void MainWidow::update_name(QList<Ups*>::Iterator it) {
  int tab = tablistaUPS->indexOf((*it)->getTabWidget());
  SQL::UpsInfo info = (*it)->getInfo();
  tablistaUPS->setTabText(tab, info.nome + QString(" - ") + info.host);
  dynamic_cast<MiniUps*>((*it)->getMiniUpsWidget())->setNome(info.nome + QString(" - ") + info.host);
  storico->aggiornaListaUps();
}

void MainWidow::menu_cancella_ups() {
  CancellaUps *d = new CancellaUps(upsList, this);
  connect(d, SIGNAL(cancella(QList<Ups*>::Iterator)), SLOT(cancella_ups(QList<Ups*>::Iterator)));
  d->show();
}

void MainWidow::cancella_ups(QList<Ups*>::Iterator it) {
  (*it)->stop();
  (*it)->waitStop();
  SQL::cancellaUps((*it)->getInfo().codups, db);
  storico->aggiornaListaUps();
  riassunto->removeWidget((*it)->getMiniUpsWidget());
  delete (*it)->getMiniUpsWidget();
  QTabWidget *tab = (*it)->getTabWidget();
  tablistaUPS->removeTab(tablistaUPS->indexOf(tab));
  delete tab;
  delete *it;
  upsList.erase(it);
}

void MainWidow::nuovo_ups(const QString & i, unsigned int porta, const QString & nome) {
  SQL::UpsInfo info;
  info.host = i;
  info.porta = porta;
  info.nome = nome;
  info.codups = SQL::salvaUps(nome, i, porta, db);
  info.maxvalori = 200;
  Ups *combo = new Ups(info);
  combo->setDB(&db);
  genera_intervalli_standard(info.codups);
  crea_gui_ups(*combo);
  combo->start();
  upsList.push_back(combo);
  storico->aggiornaListaUps();
}

void MainWidow::crea_gui_ups(Ups & u) {
  QTabWidget *tabwidget = new QTabWidget(tablistaUPS);
  tablistaUPS->addTab(tabwidget, u.getInfo().nome + QString(" - ") + u.getInfo().host);
  ListaBarre *w = new ListaBarre(u.getInfo().codups, &db, tabwidget);
  tabwidget->addTab(w, tr("Grafici a barre"));
  ListaGrafici *g = new ListaGrafici(u.getInfo().codups, &db, tabwidget);
  tabwidget->addTab(g, tr("Grafici normali"));
  u.setTabWidget(tabwidget);
  u.setWidgetBarre(w);
  u.setWidgetGrafici(g);
  w->setNumeroBarre(numerobarre);
  w->setSfumatura(sfumatura);
  w->setSfumaturaIntervalli(sfumaturaIntervalli);
  g->setSfumatura(sfumatura);
  g->setSfumaturaIntervalli(sfumaturaIntervalli);
  connect(&u, SIGNAL(nuovoValore(SQL::ValoriInfo)), w, SLOT(aggiorna(SQL::ValoriInfo)));
  connect(&u, SIGNAL(nuovoValore(SQL::ValoriInfo)), g, SLOT(aggiorna(SQL::ValoriInfo)));
  connect(&u, SIGNAL(nuoveCostanti(SQL::ValoriCostantiInfo)), w, SLOT(aggiornaCostanti(SQL::ValoriCostantiInfo)));
  connect(&u, SIGNAL(nuoveCostanti(SQL::ValoriCostantiInfo)), g, SLOT(aggiornaCostanti(SQL::ValoriCostantiInfo)));
  AllInfo *a = new AllInfo(tabwidget);
  tabwidget->addTab(a, tr("Valori apcupsd"));
  connect(&u, SIGNAL(nuovoGrezzo(QString)), a, SLOT(aggiornaTXT(QString)));
  AllEvents *e = new AllEvents(tabwidget);
  tabwidget->addTab(e, tr("Eventi apcupsd"));
  connect(&u, SIGNAL(nuoviEventi(QStringList)), e, SLOT(aggiornaTXT(QStringList)));
  connect(e, SIGNAL(askEvents()), &u, SLOT(askEvents()));
  MiniUps *mini = new MiniUps(riassunto);
  mini->setNome(u.getInfo().nome + QString(" - ") + u.getInfo().host);
  riassunto->addWidget(mini);
  u.setMiniUpsWidget(mini);
  connect(&u, SIGNAL(nuovoValore(SQL::ValoriInfo)), mini, SLOT(aggiornaTXT(SQL::ValoriInfo)));
}

void MainWidow::genera_intervalli_standard(int codups) {
  int codwidget;
  Intervallo *inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::LOADPCT).nome, true, global::BLU, db);
  inter = new Intervallo(global::VERDE, 0, 80);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 80, 100);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::LINEV).nome, true, global::BLU, db);
  inter = new Intervallo(global::ROSSO, 200, 250);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 210, 240);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::VERDE, 225, 235);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::OUTPUTV).nome, true, global::BLU, db);
  inter = new Intervallo(global::ROSSO, 200, 250);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 210, 240);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::VERDE, 225, 235);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::LINEFREQ).nome, true, global::BLU, db);
  inter = new Intervallo(global::ROSSO, 45, 55);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::VERDE, 49, 51);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::ITEMP).nome, true, global::BLU, db);
  inter = new Intervallo(global::ROSSO, 0, 60);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::ARANCIO, 10, 50);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 15, 40);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::VERDE, 20, 33);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::BCHARGE).nome, true, global::BLU, db);
  inter = new Intervallo(global::ROSSO, 0, 20);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::ARANCIO, 20, 30);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 30, 50);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::VERDE, 50, 100);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::BATTV).nome, true, global::BLU, db);
  inter = new Intervallo(global::ARANCIO, 0, 20);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::TIMELEFT).nome, true, global::BLU, db);
  inter = new Intervallo(global::ROSSO, 0, 3);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::ARANCIO, 3, 5);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 5, 7);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::VERDE, 7, 20);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;

  codwidget = SQL::salvaWidget(codups, global::codifica(global::Potenza).nome, true, global::BLU, db);
  inter = new Intervallo(global::VERDE, 0, 350);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
  inter = new Intervallo(global::GIALLO, 350, 400);
  SQL::salvaIntervallo(*inter, codwidget, db);
  delete inter;
}

