#include "modificaintervalliups.h"

ModificaIntervalliUPS::ModificaIntervalliUPS(QList<Ups*> & lista, QSqlDatabase *d, QWidget *parent) :
    QDialog(parent), upsList(lista) {
  db = d;
  setAttribute(Qt::WA_DeleteOnClose);
  setModal(Qt::WindowModal);
  setWindowTitle(tr("Impostazioni degli intervalli degli UPS"));

  v_min = false;
  v_max = false;
  v_al = false;
  sfumatura = true;
  sfumaturaIntervalli = true;
  numerobarre = 7;

  QGridLayout *griglia = new QGridLayout(this);
  QLabel *l;

  l = new QLabel(tr("Scelta dell'UPS da modificare") + QChar(':'), this);
  griglia->addWidget(l, 0, 0, 1, 4);

  ups = new QComboBox(this);
  griglia->addWidget(ups, 1, 0, 1, 4);
  for(int i = 0; i < upsList.size(); i++)
    ups->addItem(upsList[i]->getInfo().nome + QString(" - ") + upsList[i]->getInfo().host, upsList[i]->getInfo().codups);
  connect(ups, SIGNAL(currentIndexChanged(int)), SLOT(cambia_ups()));

  l = new QLabel(tr("Scelta del valore da modificare") + QChar(':'), this);
  griglia->addWidget(l, 2, 0, 1, 4);

  valoriups = new QComboBox(this);
  griglia->addWidget(valoriups, 3, 0, 1, 3);
  valoriups->addItem(global::CaricoUPS, global::LOADPCT);
  valoriups->addItem(global::TensioneLinea, global::LINEV);
  valoriups->addItem(global::TensioneUscita, global::OUTPUTV);
  valoriups->addItem(global::FrequenzaLinea, global::LINEFREQ);
  valoriups->addItem(global::TemperaturaUPS, global::ITEMP);
  valoriups->addItem(global::CaricaBatteria, global::BCHARGE);
  valoriups->addItem(global::TensioneBatteria, global::BATTV);
  valoriups->addItem(global::TempoRimasto, global::TIMELEFT);
  valoriups->addItem(global::PotenzaErogata, global::Potenza);
  connect(valoriups, SIGNAL(currentIndexChanged(int)), SLOT(cambia_valore_ups(int)));

  visibile = new QCheckBox(tr("Visibile"), this);
  griglia->addWidget(visibile, 3, 3, 1, 1);

  tabellaintervallini = new QTableWidget(this);
  griglia->addWidget(tabellaintervallini, 4, 0, 6, 1, Qt::AlignTop);
  tabellaintervallini->setColumnCount(4);
  tabellaintervallini->setColumnWidth(3, 150);
  tabellaintervallini->setMinimumWidth(500);
  tabellaintervallini->setSelectionMode(QTableWidget::SingleSelection);
  tabellaintervallini->setSelectionBehavior(QTableWidget::SelectRows);
  tabellaintervallini->setHorizontalHeaderItem(0, new QTableWidgetItem(altezzaTxt));
  tabellaintervallini->setHorizontalHeaderItem(1, new QTableWidgetItem(lim_sxTxt));
  tabellaintervallini->setHorizontalHeaderItem(2, new QTableWidgetItem(lim_dxTxt));
  tabellaintervallini->setHorizontalHeaderItem(3, new QTableWidgetItem(coloreTxt));

  l = new QLabel(altezzaTxt + QChar(':'), this);
  griglia->addWidget(l, 4, 1, 1, 1, Qt::AlignTop);
  l = new QLabel(lim_sxTxt + QChar(':'), this);
  griglia->addWidget(l, 4, 2, 1, 1, Qt::AlignTop);
  l = new QLabel(lim_dxTxt + QChar(':'), this);
  griglia->addWidget(l, 4, 3, 1, 1, Qt::AlignTop);

  livello = new QLineEdit(this);
  griglia->addWidget(livello, 5, 1, 1, 1, Qt::AlignTop);
  connect(livello, SIGNAL(textChanged(QString)), SLOT(testa_altezza(QString)));
  valoreminimo = new QLineEdit(this);
  griglia->addWidget(valoreminimo, 5, 2, 1, 1, Qt::AlignTop);
  connect(valoreminimo, SIGNAL(textChanged(QString)), SLOT(testa_minimo(QString)));
  valoremassimo = new QLineEdit(this);
  griglia->addWidget(valoremassimo, 5, 3, 1, 1, Qt::AlignTop);
  connect(valoremassimo, SIGNAL(textChanged(QString)), SLOT(testa_massimo(QString)));

  l = new QLabel(coloreTxt + QChar(':'), this);
  griglia->addWidget(l, 6, 1, 1, 3, Qt::AlignBottom);

  colore = new PrendiColore(this);
  griglia->addWidget(colore, 7, 1, 1, 3, Qt::AlignTop);

  aggiungi = new QPushButton(tr("A&ggiungi micro intervallo"), this);
  aggiungi->setEnabled(false);
  griglia->addWidget(aggiungi, 8, 1, 1, 3, Qt::AlignTop);
  connect(aggiungi, SIGNAL(clicked()), SLOT(aggiungi_intervallo()));

  rimuovi = new QPushButton(tr("&Cancella micro intervallo"), this);
  griglia->addWidget(rimuovi, 10, 0, 1, 1, Qt::AlignTop);
  connect(rimuovi, SIGNAL(clicked()), SLOT(cancella_intervallo()));

  l = new QLabel(tr("Colore delle barre") + QChar(':'), this);
  griglia->addWidget(l, 9, 1, 1, 3, Qt::AlignBottom);

  colorebarra = new PrendiColore(this);
  griglia->addWidget(colorebarra, 10, 1, 2, 3, Qt::AlignTop);
  connect(colorebarra, SIGNAL(nuovoColore(QColor)), SLOT(cambia_colore_barre(QColor)));

  l = new QLabel(tr("Anteprima con numeri casuali") + QChar(':'), this);
  griglia->addWidget(l, 11, 0, 1, 1, Qt::AlignBottom);

  anteprima = new WidgetBarra(this);
  griglia->addWidget(anteprima, 12, 0, 1, 4);
  anteprima->setNumeroBarre(numerobarre);
  anteprima->setSfumatura(sfumatura);

  annulla = new QPushButton(tr("&Annulla i cambiamenti a questo parametro"), this);
  griglia->addWidget(annulla, 13, 0, 1, 1);
  connect(annulla, SIGNAL(clicked()), SLOT(annulla_modifiche()));

  salva = new QPushButton(tr("&Salva i cambiamenti a questo parametro"), this);
  griglia->addWidget(salva, 13, 1, 1, 3);
  connect(salva, SIGNAL(clicked()), SLOT(salva_modifiche()));

  cambia_valore_ups(valoriups->currentIndex());

  connect(&generatore, SIGNAL(timeout()), SLOT(genera_casuali()));
  generatore.start(700);
  setLayout(griglia);
}

const QString ModificaIntervalliUPS::altezzaTxt(tr("Altezza"));
const QString ModificaIntervalliUPS::lim_dxTxt(tr("Limite destro"));
const QString ModificaIntervalliUPS::lim_sxTxt(tr("Limite sinistro"));
const QString ModificaIntervalliUPS::coloreTxt(tr("Colore dell'intervallo"));

void ModificaIntervalliUPS::genera_casuali() {
  QList<Intervallo> intervalli = anteprima->getIntervalli();
  if(intervalli.size() >= 1) {
    double min = intervalli[0].getSotto();
    double max = intervalli[0].getSopra();
    for(int i = 1; i < intervalli.size(); i++) {
      if(intervalli[i].getSotto() < min)
        min = intervalli[i].getSotto();
      if(intervalli[i].getSopra() > max)
        max = intervalli[i].getSopra();
    }
    double val = qAbs(max - min) * ((double)qrand() / RAND_MAX) + min;
    anteprima->pushValore(val);
  }
}

void ModificaIntervalliUPS::cambia_valore_ups(int i) {
  if(upsList.size() > 0) {
    global::ValoriUps valore = static_cast<global::ValoriUps>(valoriups->itemData(i).toInt());
    global::ValoreSingolo const &valoreCodificato = global::codifica(valore);
    anteprima->setTesto(valoreCodificato.udmTxt, valoreCodificato.udm);
    ListaWidgetAdIntervalli *widgetadintervalli = upsList[ups->currentIndex()]->getWidgetBarre();
    visibile->setChecked(widgetadintervalli->getVisibile(valore));
    WidgetBarra *widgetbarra = dynamic_cast<WidgetBarra*>(widgetadintervalli->getWidget(valore));
    anteprima->cancellaIntervalli();
    anteprima->aggiungiIntervalli(widgetbarra->getIntervalli());
    anteprima->setColoreValori(widgetbarra->getColoreValori());
    colorebarra->setColore(widgetbarra->getColoreValori());
    anteprima->update();
    aggiorna_tabella();
  }
}

void ModificaIntervalliUPS::aggiorna_tabella() {
  tabellaintervallini->clear();
  tabellaintervallini->setHorizontalHeaderItem(0, new QTableWidgetItem(altezzaTxt));
  tabellaintervallini->setHorizontalHeaderItem(1, new QTableWidgetItem(lim_sxTxt));
  tabellaintervallini->setHorizontalHeaderItem(2, new QTableWidgetItem(lim_dxTxt));
  tabellaintervallini->setHorizontalHeaderItem(3, new QTableWidgetItem(coloreTxt));
  QList<Intervallo> intervalli = anteprima->getIntervalli();
  tabellaintervallini->setRowCount(intervalli.size());
  for(int i = 0; i < intervalli.size(); i++) {
    tabellaintervallini->setCellWidget(i, 0, new QLabel(QString::number(intervalli[i].getLivello()), tabellaintervallini));
    tabellaintervallini->setCellWidget(i, 1, new QLabel(QString::number(intervalli[i].getSotto()), tabellaintervallini));
    tabellaintervallini->setCellWidget(i, 2, new QLabel(QString::number(intervalli[i].getSopra()), tabellaintervallini));
    tabellaintervallini->setCellWidget(i, 3, new Quadrattino(intervalli[i].getColore(), tabellaintervallini));
  }
}

void ModificaIntervalliUPS::cambia_ups() {
  cambia_valore_ups(valoriups->currentIndex());
}

void ModificaIntervalliUPS::annulla_modifiche() {
  cambia_valore_ups(valoriups->currentIndex());
}

void ModificaIntervalliUPS::testa_altezza(const QString &s) {
  s.toInt(&v_al);
  aggiungi->setEnabled(v_al && v_max && v_min);
}

void ModificaIntervalliUPS::testa_massimo(const QString &s) {
  s.toDouble(&v_max);
  aggiungi->setEnabled(v_al && v_max && v_min);
}

void ModificaIntervalliUPS::testa_minimo(const QString &s) {
  s.toDouble(&v_min);
  aggiungi->setEnabled(v_al && v_max && v_min);
}

void ModificaIntervalliUPS::cancella_intervallo() {
  int riga = tabellaintervallini->currentRow();
  if(riga >= 0) {
    QList<Intervallo> intervalli = anteprima->getIntervalli();
    QList<Intervallo>::Iterator it = intervalli.begin();
    it += riga;
    intervalli.erase(it);
    anteprima->cancellaIntervalli();
    anteprima->aggiungiIntervalli(intervalli);
    anteprima->update();
    aggiorna_tabella();
  }
}

void ModificaIntervalliUPS::cambia_colore_barre(const QColor & c) {
  anteprima->setColoreValori(c);
  anteprima->update();
}

void ModificaIntervalliUPS::aggiungi_intervallo() {
  Intervallo i(colore->getColore(), valoreminimo->text().toDouble(),
               valoremassimo->text().toDouble(), livello->text().toInt());
  i.setSfumatura(sfumaturaIntervalli);
  anteprima->aggiungiIntervallo(i);
  anteprima->update();
  aggiorna_tabella();
}

void ModificaIntervalliUPS::setParametriGlobali(bool s, bool si, int n) {
  sfumatura = s;
  sfumaturaIntervalli = si;
  numerobarre = n;
  anteprima->setNumeroBarre(n);
  anteprima->setSfumatura(s);
  anteprima->setSfumaturaIntervalli(si);
  anteprima->update();
}

void ModificaIntervalliUPS::salva_modifiche() {
  if(upsList.size() > 0) {
    global::ValoriUps v = static_cast<global::ValoriUps>(valoriups->itemData(valoriups->currentIndex()).toInt());
    ListaWidgetAdIntervalli *widgetbarre = upsList[ups->currentIndex()]->getWidgetBarre();
    ListaWidgetAdIntervalli *widgetgrafici = upsList[ups->currentIndex()]->getWidgetGrafici();
    widgetbarre->setVisible(visibile->isChecked(), v);
    widgetgrafici->setVisible(visibile->isChecked(), v);
    WidgetAdIntervalli *barra = widgetbarre->getWidget(v);
    WidgetAdIntervalli *grafico = widgetgrafici->getWidget(v);
    barra->setColoreValori(anteprima->getColoreValori());
    grafico->setColoreValori(anteprima->getColoreValori());
    barra->cancellaIntervalli();
    grafico->cancellaIntervalli();
    barra->aggiungiIntervalli(anteprima->getIntervalli());
    grafico->aggiungiIntervalli(anteprima->getIntervalli());
    widgetbarre->salvaWidget(v);
  }
}
