#include "networkthread.h"

NetworkThread::NetworkThread(unsigned int msec, QObject *parent) :
    QThread(parent), intervallo(msec),
    indirizzo(QHostAddress::LocalHost), porta(3551) {
  cicla = true;
  getEvents = false;
}

NetworkThread::~NetworkThread() {
  if(isRunning()) {
    termina();
    wait();
  }
}

SQL::ValoriInfo NetworkThread::getNewValue() {
  SQL::ValoriInfo r;
  lock();
  r = memVal;
  unlock();
  return r;
}

SQL::ValoriCostantiInfo NetworkThread::getNewConstant() {
  SQL::ValoriCostantiInfo r;
  lock();
  r = memCons;
  unlock();
  return r;
}

QString NetworkThread::getNewAPCStatus() {
  QString r;
  lock();
  r = allStatus;
  unlock();
  return r;
}

QStringList NetworkThread::getNewAPCEvents() {
  QStringList r;
  lock();
  r = allEvents;
  unlock();
  return r;
}

void NetworkThread::setIndirizzo(const QString & add, unsigned int p) {
  bool run;

  run = isRunning();
  if(run) {
    termina();
    wait();
  }

  indirizzo = add;
  porta = p;

  if(run) {
    cicla = true;
    start();
  }
}

void NetworkThread::setIntervallo(unsigned int msec) {
  lock();
  intervallo = msec;
  unlock();
}

void NetworkThread::askEvents() {
  lock();
  getEvents = true;
  unlock();
}

bool NetworkThread::testEnel(const QString & STATUS) {
  if(STATUS == QString("ONLINE") || STATUS == QString("OK"))
    return true;
  else
    return false;
}

float NetworkThread::getFloat(const QString & s) {
  QString val;
  for(int i = 0; i < s.size(); i++)
    if(s[i].isDigit() || s[i] == '.')
      val.push_back(s[i]);
  bool ok;
  float ritorna = val.toFloat(&ok);
  if(ok)
    return ritorna;
  else
    return 0;
}

QString NetworkThread::getString(const QString & s) {
  QString val;
  int i = 0;
  int j = s.size() - 1;
  while(i < s.size() && s[i] == ' ')
    i++;
  while(j >= 0 && (s[j] == QChar(' ') || s[j] == QChar('\n') || s[j] == QChar('\r')))
    j--;
  while(i <= j) {
    val.push_back(s[i]);
    i++;
  }
return val;
}

QDate NetworkThread::getDate(const QString & s) {
  int yyyy = 0, mm = 0, dd = 0;
  int i = 0;
  int pos  = 0;
  QString ss = getString(s);
  while(i < ss.size()) {
    static const QChar char1('-');
    if(ss[i] == char1)
      pos++;
    else if(ss[i].isDigit()) {
      if(pos == 0)
        yyyy = yyyy * 10 + ss[i].digitValue();
      else if(pos == 1)
        mm = mm * 10 + ss[i].digitValue();
      else
        dd = dd * 10 + ss[i].digitValue();
    }
    i++;
  }
  return QDate(yyyy, mm, dd);
}

QDateTime NetworkThread::getDateTime(const QString & s) {
  int yyyy = 0, mm = 0, dd = 0, h = 0, m = 0, sec = 0;
  int i = 0;
  int pos  = 0;
  QString ss = getString(s);
  QString buffer;
  bool normale = false;
  while(i < ss.size() + 1) {
    static const QChar char1(' ');
    static const QChar char2(':');
    static const QChar char3('-');
    if(i == ss.size() || ss[i] == char1 || ss[i] == char2 || ss[i] == char3) {
      pos++;
      if(pos == 1)
        yyyy = buffer.toInt(&normale);
      if(normale) {
        switch(pos) {
          case 2: mm = buffer.toInt(); break;
          case 3: dd = buffer.toInt(); break;
          case 4: h = buffer.toInt(); break;
          case 5: m = buffer.toInt(); break;
          case 6: sec = buffer.toInt(); break;
        }
      } else {
        if(pos == 2) {
          if(buffer == "Jan") mm = 1; else
          if(buffer == "Feb") mm = 2; else
          if(buffer == "Mar") mm = 3; else
          if(buffer == "Apr") mm = 4; else
          if(buffer == "May") mm = 5; else
          if(buffer == "Jun") mm = 6; else
          if(buffer == "Jul") mm = 7; else
          if(buffer == "Aug") mm = 8; else
          if(buffer == "Sep") mm = 9; else
          if(buffer == "Oct") mm = 10; else
          if(buffer == "Nov") mm = 11; else
          if(buffer == "Dec") mm = 12;
        } else if(pos == 3) dd = buffer.toInt();
        else if(pos == 4) h = buffer.toInt();
        else if(pos == 5) m = buffer.toInt();
        else if(pos == 6) sec = buffer.toInt();
        else if(pos == 8) yyyy = buffer.toInt();
      }
      buffer.clear();
    } else
      buffer.push_back(ss[i]);
    i++;
  }

  return QDateTime(QDate(yyyy, mm, dd), QTime(h, m, sec, 0));
}

QStringList NetworkThread::convertEvents(const QString &s) {
  QString row;
  QStringList events;
  for(int i = 0; i < s.size(); i++) {
    static const QChar char1('\n');
    static const QChar char2('\r');
    if(s[i] == char1) {
      events.push_front(row);
      row.clear();
    } else if(s[i] != char2)
      row.push_back(s[i]);
  }
  if(row.size() > 0)
    events.push_front(row);
  return events;
}

void NetworkThread::run() {
  char recvline[5000];
  QString output;
  QTcpSocket statusSocket, eventsSocket;
  QRegExp exp("^APC *:.*\nEND APC *:[^\n]+\n$");

  while(cicla) {
    output.clear();

    if(statusSocket.state() == QTcpSocket::UnconnectedState) {
      statusSocket.connectToHost(indirizzo, porta);
      if(statusSocket.waitForConnected(5000))
        statusSocket.open(QTcpSocket::ReadWrite);
      else
        statusSocket.abort();
    }

    if(statusSocket.state() == QTcpSocket::ConnectedState) {
      net_write(&statusSocket, "status", 6);
      statusSocket.waitForBytesWritten(5000);

      int n = 1;
      while(cicla && n > 0) {
        n = net_read(&statusSocket, recvline, sizeof(recvline));
        if(n > 0) {
          recvline[n] = 0;
          output.append(recvline);
        }
      }

      if(cicla && exp.exactMatch(output)) {
        lock();
        converti(output, memVal, memCons);
        allStatus = output;
        unlock();
        emit newValues();
        emit newConstants();
      }

      lock();
      if(getEvents && eventsSocket.state() == QTcpSocket::UnconnectedState) {
        eventsSocket.connectToHost(indirizzo, porta);
        if(eventsSocket.waitForConnected(5000))
          eventsSocket.open(QTcpSocket::ReadWrite);
        else
          eventsSocket.abort();
        getEvents = false;
      }
      unlock();

      if(eventsSocket.state() == QTcpSocket::ConnectedState) {
        net_write(&eventsSocket, "events", 6);
        eventsSocket.waitForBytesWritten(5000);
        output.clear();

        n = 1;
        while(cicla && n > 0) {
          n = net_read(&eventsSocket, recvline, sizeof(recvline));
          if(n > 0) {
            recvline[n] = 0;
            output.append(recvline);
          }
        }

        lock();
        allEvents = convertEvents(output);
        unlock();
        emit newEvents();

        eventsSocket.disconnectFromHost();
      }
    }

    for(unsigned int i = 0; i < intervallo && cicla; i += 10)
      msleep(10);
  }

  if(statusSocket.state() == QTcpSocket::ConnectedState)
    statusSocket.disconnectFromHost();
}

int NetworkThread::net_read(QTcpSocket *s, char *buffer, qint16 maxlen) {
  int received;
  qint16 pktsize;

  if(s->bytesAvailable() < 2)
    s->waitForReadyRead(5000);
  received = s->read(reinterpret_cast<char *>(&pktsize), sizeof(pktsize));

  if(received <= 0)
    return received;
  if(received != sizeof(qint16))
    return -1;

  pktsize = net_to_host(pktsize);
  if(pktsize > maxlen)
    return -1;
  if(pktsize == 0)
    return 0;

  if(s->bytesAvailable() < pktsize)
    s->waitForReadyRead(500);

  received = s->read(buffer, pktsize);
  if(received <= 0)
    return received;
  if(received != pktsize)
    return -1;

  return received;
}

int NetworkThread::net_write(QTcpSocket *s, const char *buffer, qint16 len) {
  int send;
  qint16 pktsize;

  pktsize = host_to_net(len);
  send = s->write(reinterpret_cast<char *>(&pktsize), sizeof(pktsize));
  if(send <= 0)
    return send;
  if(send != sizeof(qint16))
    return -1;

  send = s->write(buffer, len);
  if(send <= 0)
    return send;
  if(send != len)
    return -1;

  return send;
}

void NetworkThread::termina() {
  cicla = false;
}

qint16 NetworkThread::net_to_host(qint16 s) {
  qSwap(*(reinterpret_cast<qint8 *>(&s)),*(reinterpret_cast<qint8 *>(&s) + 1));
  return s;
}

qint16 NetworkThread::host_to_net(qint16 s) {
  return net_to_host(s);
}

void NetworkThread::converti(const QString & s, SQL::ValoriInfo & val, SQL::ValoriCostantiInfo & constval) {
  QString parametro;
  QString valore;
  bool duepunti = false;
  bool newline = true;
  static const QChar char1('\n');
  static const QChar char2(':');
  for(int i = 0; i < s.size() + 1; i++) {
    if(i >= s.size() || s[i] == char1) {
      newline = true;
      duepunti = false;
      parametro = getString(parametro);
      if(parametro == global::LOADPCTtxt) val.LOADPCT = getFloat(valore); else
      if(parametro == global::LINEVtxt) val.LINEV = getFloat(valore); else
      if(parametro == global::OUTPUTVtxt) val.OUTPUTV = getFloat(valore); else
      if(parametro == global::LINEFREQtxt) val.LINEFREQ = getFloat(valore); else
      if(parametro == global::ITEMPtxt) val.ITEMP = getFloat(valore); else
      if(parametro == global::BCHARGEtxt) val.BCHARGE = getFloat(valore); else
      if(parametro == global::BATTVtxt) val.BATTV = getFloat(valore); else
      if(parametro == global::TIMELEFTtxt) val.TIMELEFT = getFloat(valore); else
      if(parametro == global::MBATTCHGtxt) constval.MBATTCHG = getFloat(valore); else
      if(parametro == global::MINTIMELtxt) constval.MINTIMEL = static_cast<char>(getFloat(valore)); else
      if(parametro == global::LOTRANStxt) constval.LOTRANS = getFloat(valore); else
      if(parametro == global::HITRANStxt) constval.HITRANS = getFloat(valore); else
      if(parametro == global::NOMOUTVtxt) constval.NOMOUTV = getFloat(valore); else
      if(parametro == global::NOMINVtxt) constval.NOMINV = getFloat(valore); else
      if(parametro == global::NOMBATTVtxt) constval.NOMBATTV = getFloat(valore); else
      if(parametro == global::NOMPOWERtxt) constval.NOMPOWER = getFloat(valore); else
      if(parametro == global::DATEtxt) val.DATE = getDateTime(getString(valore)); else
      if(parametro == global::HOSTNAMEtxt) constval.HOSTNAME = getString(valore); else
      if(parametro == global::RELEASEtxt) constval.RELEASE = getString(valore); else
      if(parametro == global::UPSNAMEtxt) constval.UPSNAME = getString(valore); else
      if(parametro == global::CABLEtxt) constval.CABLE = getString(valore); else
      if(parametro == global::STATUStxt) val.STATUS = testEnel(getString(valore)); else
      if(parametro == global::MODELtxt) constval.MODEL = getString(valore); else
      if(parametro == global::SENSEtxt) constval.SENSE = getString(valore); else
      if(parametro == global::STATFLAGtxt) constval.STATFLAG = getString(valore); else
      if(parametro == global::SERIALNOtxt) constval.SERIALNO = getString(valore); else
      if(parametro == global::BATTDATEtxt) constval.BATTDATE = getDate(getString(valore)); else
      if(parametro == global::FIRMWAREtxt) constval.FIRMWARE = getString(valore);
      parametro.clear();
      valore.clear();
      continue;
    }

    if(s[i] == char2 && newline) {
      newline = false;
      duepunti = true;
      continue;
    }

    if(newline)
      parametro.push_back(s[i]);
    else
      valore.push_back(s[i]);
  }
}
