#ifndef NETWORKTHREAD_H
#define NETWORKTHREAD_H

#include <QtCore>
#include <QtNetwork>
#include "sql.h"
#include "global.h"

class NetworkThread : public QThread, private QMutex {
  Q_OBJECT

  public:
    NetworkThread(unsigned int msec = 1000, QObject *parent = 0);
    ~NetworkThread();

    void setIntervallo(unsigned int msec);
    void setIndirizzo(const QString & add, unsigned int p = 3551);
    void termina();
    void askEvents();
    SQL::ValoriInfo getNewValue();
    SQL::ValoriCostantiInfo getNewConstant();
    QString getNewAPCStatus();
    QStringList getNewAPCEvents();

  protected:
    void run();

  private:
    unsigned int intervallo;
    QString indirizzo;
    unsigned int porta;
    bool cicla;
    bool getEvents;
    SQL::ValoriInfo memVal;
    SQL::ValoriCostantiInfo memCons;
    QString allStatus;
    QStringList allEvents;

    int net_write(QTcpSocket *s, const char *buffer, qint16 len);
    int net_read(QTcpSocket *s, char *buffer, qint16 maxlen);
    qint16 net_to_host(qint16 s);
    qint16 host_to_net(qint16 s);

    void converti(const QString & s, SQL::ValoriInfo & val, SQL::ValoriCostantiInfo & constval);
    QStringList convertEvents(const QString & s);
    bool testEnel(const QString & STATUS);

    static float getFloat(const QString & s);
    static QString getString(const QString & s);
    static QDate getDate(const QString & s);
    static QDateTime getDateTime(const QString & s);

  signals:
    void newValues();
    void newConstants();
    void newEvents();
};

#endif // NETWORKTHREAD_H
