#include "sql.h"

SQL::WidgetInfo::WidgetInfo() {
  codwidget = 0;
  visibile = false;
  colore = Qt::blue;
}

SQL::UpsInfo::UpsInfo() {
  porta = 0;
  codups = 0;
  maxvalori = 0;
}

SQL::ValoriInfo::ValoriInfo() {
  LOADPCT = 0;
  LINEV = 0;
  OUTPUTV = 0;
  LINEFREQ = 0;
  ITEMP = 0;
  BCHARGE = 0;
  BATTV = 0;
  TIMELEFT = 0;
  STATUS = false;
}

bool SQL::ValoriInfo::isEquivalent(const ValoriInfo & v) const {
  return LOADPCT == v.LOADPCT &&
         LINEV == v.LINEV &&
         OUTPUTV == v.OUTPUTV &&
         LINEFREQ == v.LINEFREQ &&
         ITEMP == v.ITEMP &&
         BCHARGE == v.BCHARGE &&
         BATTV == v.BATTV &&
         TIMELEFT == v.TIMELEFT &&
         STATUS == v.STATUS;
}

bool SQL::ValoriInfo::operator==(const SQL::ValoriInfo & v) const {
  return DATE == v.DATE;
}

bool SQL::ValoriInfo::operator!=(const SQL::ValoriInfo & v) const {
  return DATE != v.DATE;
}

bool SQL::ValoriInfo::operator>=(const SQL::ValoriInfo & v) const {
  return DATE >= v.DATE;
}

bool SQL::ValoriInfo::operator<=(const SQL::ValoriInfo & v) const {
  return DATE <= v.DATE;
}

bool SQL::ValoriInfo::operator>(const SQL::ValoriInfo & v) const {
  return DATE > v.DATE;
}

bool SQL::ValoriInfo::operator<(const SQL::ValoriInfo & v) const {
  return DATE < v.DATE;
}

SQL::ValoriCostantiInfo::ValoriCostantiInfo() {
  MINTIMEL = 0;
  LOTRANS = 0;
  HITRANS = 0;
  NOMOUTV = 0;
  NOMINV = 0;
  NOMBATTV = 0;
  NOMPOWER = 0;
}

bool SQL::ValoriCostantiInfo::operator==(const SQL::ValoriCostantiInfo & v) {
  return MBATTCHG == v.MBATTCHG &&
         MINTIMEL == v.MINTIMEL &&
         LOTRANS == v.LOTRANS &&
         HITRANS == v.HITRANS &&
         NOMOUTV == v.NOMOUTV &&
         NOMINV == v.NOMINV &&
         NOMBATTV == v.NOMBATTV &&
         NOMPOWER == v.NOMPOWER &&
         HOSTNAME == v.HOSTNAME &&
         RELEASE == v.RELEASE &&
         UPSNAME == v.UPSNAME &&
         CABLE == v.CABLE &&
         MODEL == v.MODEL &&
         SENSE == v.SENSE &&
         STATFLAG == v.STATFLAG &&
         SERIALNO == v.SERIALNO &&
         BATTDATE == v.BATTDATE &&
         FIRMWARE == v.FIRMWARE;
}

bool SQL::ValoriCostantiInfo::operator!=(const SQL::ValoriCostantiInfo & v) {
  return !operator==(v);
}

QString SQL::makeDate(const QDate & d) {
  return makeDate(QDateTime(d));
}

QString SQL::makeDate(const QDateTime & d) {
  QString ritorna;
  ritorna += QString::number(d.date().year()) + QChar('-');
  if(d.date().month() < 10) ritorna += QChar('0');
  ritorna += QString::number(d.date().month()) + QChar('-');
  if(d.date().day() < 10) ritorna += QChar('0');
  ritorna += QString::number(d.date().day()) + QChar(' ');
  if(d.time().hour() < 10) ritorna += QChar('0');
  ritorna += QString::number(d.time().hour()) + QChar(':');
  if(d.time().minute() < 10) ritorna += QChar('0');
  ritorna += QString::number(d.time().minute()) + QChar(':');
  if(d.time().second() < 10) ritorna += QChar('0');
  ritorna += QString::number(d.time().second());
  return ritorna;
}

QString SQL::cleanQueryTxt(const QString & s) {
  QString ritorna;
  for(int i = 0; i < s.size(); i++) {
    if(s[i] == QChar('\''))
      ritorna += QString("'") + s[i];
    else if(s[i] == QChar('\n'))
      ritorna += QString("\\n");
    else if(s[i] == QChar('\r'))
      ritorna += QString("\\r");
    else if(s[i] == QChar('\t'))
      ritorna += QString("\\t");
    else if(s[i] == QChar('\\'))
      ritorna += QString("\\");
    else if(s[i] == QChar('%'))
      ritorna += QString("\\%");
    else
      ritorna += s[i];
  }
  return ritorna;
}

void SQL::attivaForeignKey(QSqlDatabase & db) {
  db.exec(QString("PRAGMA foreign_keys = ON"));
}

void SQL::creaTabelle(QSqlDatabase & db) {
  QString query;
  query = QString("create table if not exists PARAMETRI ("
                  "parametro text primary key,"
                  "valore text"
                  ")");
  db.exec(query);

  query = QString("create table if not exists UPS ("
                  "codups integer primary key autoincrement,"
                  "nome text,"
                  "indirizzo text,"
                  "porta integer,"
                  "maxvalori integer,"
                  "MBATTCHG real,"
                  "MINTIMEL char,"
                  "LOTRANS real,"
                  "HITRANS real,"
                  "NOMOUTV real,"
                  "NOMINV real,"
                  "NOMBATTV real,"
                  "NOMPOWER real,"
                  "HOSTNAME text,"
                  "RELEASE text,"
                  "UPSNAME text,"
                  "CABLE text,"
                  "MODEL text,"
                  "SENSE text,"
                  "STATFLAG text,"
                  "SERIALNO text,"
                  "BATTDATE numeric,"
                  "FIRMWARE text"
                  ")");
  db.exec(query);

  query = QString("create table if not exists WIDGET ("
                  "codwidget integer primary key autoincrement,"
                  "codups integer,"
                  "widget text,"
                  "visibile integer,"
                  "r integer,"
                  "g integer,"
                  "b integer,"
                  "foreign key(codups) references UPS(codups) on delete cascade,"
                  "unique(codups, widget)"
                  ")");
  db.exec(query);

  query = QString("create table if not exists INTERVALLI ("
                  "codcolore integer primary key autoincrement,"
                  "codwidget integer,"
                  "r integer,"
                  "g integer,"
                  "b integer,"
                  "livello integer,"
                  "sotto real,"
                  "sopra real,"
                  "foreign key(codwidget) references WIDGET(codwidget) on delete cascade"
                  ")");
  db.exec(query);

  query = QString("create table VALORI ("
                  "codvalore integer primary key autoincrement,"
                  "codups integer,"
                  "LOADPCT real,"
                  "LINEV real,"
                  "OUTPUTV real,"
                  "LINEFREQ real,"
                  "ITEMP real,"
                  "BCHARGE real,"
                  "BATTV real,"
                  "TIMELEFT real,"
                  "STATUS integer,"
                  "DATE numeric,"
                  "foreign key(codups) references UPS(codups) on delete cascade"
                  ")");
  db.exec(query);

  query = QString("create index if not exists INDICE on VALORI (DATE, codups, STATUS)");
  db.exec(query);
  }

void SQL::salvaIntervallo(const Intervallo & i, int codwindget, QSqlDatabase & db) {
  QString query("insert into INTERVALLI (codwidget, r, g, b, livello, sotto, sopra) values (");
  query += QString::number(codwindget) + QChar(',');
  query += QString::number(i.getColore().red()) + QChar(',');
  query += QString::number(i.getColore().green()) + QChar(',');
  query += QString::number(i.getColore().blue()) + QChar(',');
  query += QString::number(i.getLivello()) + QChar(',');
  query += QString::number(i.getSotto()) + QChar(',');
  query += QString::number(i.getSopra());
  query += QChar(')');
  db.exec(query);
}

void SQL::salvaIntervalli(const QList<Intervallo> & i, int codwindget, QSqlDatabase & db) {
  for(int j = 0; j < i.size(); j++)
    salvaIntervallo(i[j], codwindget, db);
}

QList<Intervallo> SQL::prendiIntervalli(int codwidget, QSqlDatabase & db) {
  QList<Intervallo> ritorna;
  QString query("select r, g, b, livello, sotto, sopra from INTERVALLI where codwidget = ");
  query += QString::number(codwidget);
  QSqlQuery q = db.exec(query);
  q.first();
  while(q.isValid()) {
    QColor c(q.value(0).toInt(), q.value(1).toInt(), q.value(2).toInt());
    Intervallo i(c, q.value(4).toDouble(), q.value(5).toDouble(), q.value(3).toInt());
    ritorna.push_back(i);
    q.next();
  }
  return ritorna;
}

void SQL::cancellaIntervalli(int codwidget, QSqlDatabase & db) {
  QString query("delete from INTERVALLI where codwidget = ");
  query += QString::number(codwidget);
  db.exec(query);
}

void SQL::salvaUpsStatus(int codups, const ValoriInfo & valori, QSqlDatabase & db) {
  QVector<ValoriInfo> v;
  v.push_back(valori);
  salvaUpsStatus(codups, v, db);
}

void SQL::salvaUpsStatus(int codups, const QVector<ValoriInfo> & valori, QSqlDatabase & db) {
  QString query;
  QString incipit("insert into VALORI ("
                  "codups , LOADPCT , LINEV , OUTPUTV , LINEFREQ ,"
                  "ITEMP , BCHARGE , BATTV , TIMELEFT , STATUS, DATE) ");
  int i = 0;
  while(i < valori.size()) {
    query = incipit;
    for(int j = 0; j < 250 && i < valori.size(); j++, i++) {
      if(j > 0)
        query += QString(" union ");
      query += QString("select ");
      query += QString::number(codups) + QChar(',');
      query += QString::number(valori[i].LOADPCT) + QChar(',');
      query += QString::number(valori[i].LINEV) + QChar(',');
      query += QString::number(valori[i].OUTPUTV) + QChar(',');
      query += QString::number(valori[i].LINEFREQ) + QChar(',');
      query += QString::number(valori[i].ITEMP) + QChar(',');
      query += QString::number(valori[i].BCHARGE) + QChar(',');
      query += QString::number(valori[i].BATTV) + QChar(',');
      query += QString::number(valori[i].TIMELEFT) + QChar(',');
      query += valori[i].STATUS ? QChar('1') : QChar('0');
      query += QChar(',');
      query += QString("julianday('") + makeDate(valori[i].DATE) + QString("')");
    }
    db.exec(query);
  }
}

void SQL::salvaUpsCostanti(int codups, const ValoriCostantiInfo & costanti, QSqlDatabase & db) {
  QString query("update UPS set ");
  query += QString("MBATTCHG = ") + QString::number(costanti.MBATTCHG) + QChar(',');
  query += QString("MINTIMEL = ") + QString::number(costanti.MINTIMEL) + QChar(',');
  query += QString("LOTRANS = ") + QString::number(costanti.LOTRANS) + QChar(',');
  query += QString("HITRANS = ") + QString::number(costanti.HITRANS) + QChar(',');
  query += QString("NOMOUTV = ") + QString::number(costanti.NOMOUTV) + QChar(',');
  query += QString("NOMINV = ") + QString::number(costanti.NOMINV) + QChar(',');
  query += QString("NOMBATTV = ") + QString::number(costanti.NOMBATTV) + QChar(',');
  query += QString("NOMPOWER = ") + QString::number(costanti.NOMPOWER) + QChar(',');
  query += QString("HOSTNAME = ") + QChar('\'') + costanti.HOSTNAME + QChar('\'') + QChar(',');
  query += QString("RELEASE = ") + QChar('\'') + costanti.RELEASE + QChar('\'') + QChar(',');
  query += QString("UPSNAME = ") + QChar('\'') + costanti.UPSNAME + QChar('\'') + QChar(',');
  query += QString("CABLE = ") + QChar('\'') + costanti.CABLE + QChar('\'') + QChar(',');
  query += QString("MODEL = ") + QChar('\'') + costanti.MODEL + QChar('\'') + QChar(',');
  query += QString("SENSE = ") +  QChar('\'') + costanti.SENSE + QChar('\'') + QChar(',');
  query += QString("STATFLAG = ") + QChar('\'') + costanti.STATFLAG + QChar('\'') + QChar(',');
  query += QString("SERIALNO = ") + QChar('\'') + costanti.SERIALNO + QChar('\'') + QChar(',');
  query += QString("BATTDATE = ") + QString("julianday('") + makeDate(costanti.BATTDATE) + QString("')") + QChar(',');
  query += QString("FIRMWARE = ") + QChar('\'') + costanti.FIRMWARE + QChar('\'');
  query += QString(" where codups = ") + QString::number(codups);
  db.exec(query);
}

void SQL::cancellaUps(int codups, QSqlDatabase & db) {
  QString query("delete from UPS where codups = ");
  query += QString::number(codups);
  db.exec(query);
}

int SQL::salvaUps(const QString & nome, const QString & host, int porta, QSqlDatabase & db) {
  QString query("insert into UPS (nome, indirizzo, porta, maxvalori) values (");
  query += QChar('\'') + cleanQueryTxt(nome) + QChar('\'') + QChar(',');
  query += QChar('\'') + cleanQueryTxt(host) + QChar('\'') + QChar(',');
  query += QString::number(porta) + QChar(',');
  query += QString("200");
  query += QChar(')');
  return db.exec(query).lastInsertId().toInt();
}

QList<SQL::UpsInfo> SQL::prendiUps(QSqlDatabase & db) {
  QList<UpsInfo> ritorna;
  QString query("select codups, nome, indirizzo, porta, maxvalori from UPS order by codups");
  QSqlQuery q1 = db.exec(query);
  q1.first();
  while(q1.isValid()) {
    UpsInfo u;
    u.codups = q1.value(0).toInt();
    u.nome = q1.value(1).toString();
    u.host = q1.value(2).toString();
    u.porta = q1.value(3).toInt();
    u.maxvalori = q1.value(4).toInt();
    ritorna.push_back(u);
    q1.next();
  }
  return ritorna;
}

QList<SQL::ValoriInfo> SQL::prendiIntervalloUpsStatus(int codups, const QDate & datainizio, const QTime & orainizio, const QDate & datafine, const QTime & orafine, QSqlDatabase & db) {
  return prendiIntervalloUpsStatus(codups, QDateTime(datainizio, orainizio), QDateTime(datafine, orafine), db);
}

QList<SQL::ValoriInfo> SQL::prendiIntervalloUpsStatus(int codups, const QDateTime & inizio, const QDateTime & fine, QSqlDatabase & db) {
  QList<ValoriInfo> ritorna;
  QString query("select LOADPCT , LINEV , OUTPUTV , LINEFREQ , "
                "ITEMP , BCHARGE , BATTV , TIMELEFT , STATUS, strftime('%Y-%m-%dT%H:%M:%S', DATE) "
                "from VALORI where DATE between "
               );
  query += QString("julianday('") + makeDate(inizio) + QString("') and ");
  query += QString("julianday('") + makeDate(fine) + QString("') ");
  query += QString("and codups = ") + QString::number(codups) + QChar(' ');
  query += QString("order by DATE");
  QSqlQuery q1 = db.exec(query);
  q1.first();
  while(q1.isValid()) {
    ValoriInfo v;
    v.LOADPCT = q1.value(0).toDouble();
    v.LINEV = q1.value(1).toDouble();
    v.OUTPUTV = q1.value(2).toDouble();
    v.LINEFREQ = q1.value(3).toDouble();
    v.ITEMP = q1.value(4).toDouble();
    v.BCHARGE = q1.value(5).toDouble();
    v.BATTV = q1.value(6).toDouble();
    v.TIMELEFT = q1.value(7).toDouble();
    v.STATUS = q1.value(8).toBool();
    v.DATE = q1.value(9).toDateTime();
    ritorna.push_back(v);
    q1.next();
  }
  return ritorna;
}

QDateTime SQL::massimaData(int codups, QSqlDatabase & db) {
  QString query("select strftime('%Y-%m-%dT%H:%M:%S', max(DATE)) from valori where codups = ");
  query += QString::number(codups);
  QSqlQuery q1 = db.exec(query);
  q1.first();
  return q1.value(0).toDateTime();
}

QDateTime SQL::minimaData(int codups, QSqlDatabase & db) {
  QString query("select strftime('%Y-%m-%dT%H:%M:%S', min(DATE)) from valori where codups = ");
  query += QString::number(codups);
  QSqlQuery q1 = db.exec(query);
  q1.first();
  return q1.value(0).toDateTime();
}

void SQL::salvaParametro(const QString &parametro, bool valore, QSqlDatabase & db) {
  salvaParametro(parametro, valore ? 1 : 0, db);
}

void SQL::salvaParametro(const QString &parametro, double valore, QSqlDatabase & db) {
  salvaParametro(parametro, QString::number(valore), db);
}

void SQL::salvaParametro(const QString &parametro, int valore, QSqlDatabase &db) {
  salvaParametro(parametro, QString::number(valore), db);
}

void SQL::salvaParametro(const QString &parametro, QString valore, QSqlDatabase &db) {
  QString query("insert or replace into PARAMETRI (parametro, valore) values (");
  query += QChar('\'') + cleanQueryTxt(parametro) + QChar('\'') + QChar(',');
  query += QChar('\'') + cleanQueryTxt(valore) + QChar('\'') + QChar(')');
  db.exec(query);
}

QVariant SQL::prendiParametro(QString parametro, QSqlDatabase & db) {
  QVariant ritorna;
  QString query("select valore from PARAMETRI where parametro = '");
  query += cleanQueryTxt(parametro) + QChar('\'');
  QSqlQuery q1 = db.exec(query);
  q1.first();
  if(q1.isValid())
    return q1.value(0);
  else
    return ritorna;
}

int SQL::salvaWidget(int codups, const QString & nome, bool visibile, const QColor & colore, QSqlDatabase & db) {
  QString query("insert into WIDGET (codups, widget, visibile, r, g, b) values (");
  query += QString::number(codups) + QChar(',');
  query += QChar('\'') + cleanQueryTxt(nome) + QChar('\'') + QChar(',');
  query += visibile ? QChar('1') : QChar('0');
  query += QChar(',') + QString::number(colore.red()) + QChar(',') + QString::number(colore.green());
  query += QChar(',') + QString::number(colore.blue());
  query += QChar(')');
  return db.exec(query).lastInsertId().toInt();
}

void SQL::cancellaWidget(int codwidget, QSqlDatabase & db) {
  QString query("delete from WIDGET where codwidget = ");
  query += QString::number(codwidget);
  db.exec(query);
}

void SQL::cancellaTuttiWidgetUps(int codups, QSqlDatabase & db) {
  QString query("delete from WIDGET where codups = ");
  query += QString::number(codups);
  db.exec(query);
}

SQL::WidgetInfo SQL::prendiWidget(int codups, const QString & nome, QSqlDatabase & db) {
  QString query("select codwidget, visibile, r, g, b from WIDGET where widget = '");
  query += cleanQueryTxt(nome) + QChar('\'');
  query += QString(" and codups = ") + QString::number(codups);
  QSqlQuery q1 = db.exec(query);
  q1.first();
  WidgetInfo ritorna;
  if(q1.isValid()) {
    ritorna.codwidget = q1.value(0).toInt();
    ritorna.visibile = q1.value(1).toBool();
    ritorna.colore.setRed(q1.value(2).toInt());
    ritorna.colore.setGreen(q1.value(3).toInt());
    ritorna.colore.setBlue(q1.value(4).toInt());
  }
  return ritorna;
}

void SQL::cancellaUpsStatusVecchi(const QDateTime & data, QSqlDatabase & db) {
  QString query("delete from VALORI where DATE <= julianday('");
  query += makeDate(data) + QString("')");
  db.exec(query);
}

void SQL::aggiornaUps(const UpsInfo & u, QSqlDatabase & db) {
  QString query("update UPS set nome = '");
  query += cleanQueryTxt(u.nome) + QString("', indirizzo = '");
  query += cleanQueryTxt(u.host) + QString("', porta = ");
  query += QString::number(u.porta) + QString(", maxvalori = ");
  query += QString::number(u.maxvalori) + QString(" where codups = ");
  query += QString::number(u.codups);
  db.exec(query);
}
