#ifndef SQL_H
#define SQL_H

#include <QtSql>
#include <QtCore>
#include "intervallo.h"

namespace SQL {
    struct WidgetInfo {
      WidgetInfo();
      int codwidget;
      bool visibile;
      QColor colore;
    };

    struct UpsInfo {
      UpsInfo();
      QString nome;
      QString host;
      int porta;
      int codups;
      int maxvalori;
    };

    struct ValoriInfo {
      ValoriInfo();
      bool operator==(const ValoriInfo & v) const;
      bool operator!=(const ValoriInfo & v) const;
      bool operator>=(const ValoriInfo & v) const;
      bool operator<=(const ValoriInfo & v) const;
      bool operator>(const ValoriInfo & v) const;
      bool operator<(const ValoriInfo & v) const;
      bool isEquivalent(const ValoriInfo & v) const;
      double LOADPCT;
      double LINEV;
      double OUTPUTV;
      double LINEFREQ;
      double ITEMP;
      double BCHARGE;
      double BATTV;
      double TIMELEFT;
      bool STATUS;
      QDateTime DATE;
    };

    struct ValoriCostantiInfo {
      ValoriCostantiInfo();
      bool operator==(const ValoriCostantiInfo & v);
      bool operator!=(const ValoriCostantiInfo & v);
      double MBATTCHG;
      double MINTIMEL;
      double LOTRANS;
      double HITRANS;
      double NOMOUTV;
      double NOMINV;
      double NOMBATTV;
      double NOMPOWER;
      QString HOSTNAME;
      QString RELEASE;
      QString UPSNAME;
      QString CABLE;
      QString MODEL;
      QString SENSE;
      QString STATFLAG;
      QString SERIALNO;
      QDate BATTDATE;
      QString FIRMWARE;
    };

    QString cleanQueryTxt(const QString & s);
    QString makeDate(const QDateTime & d);
    QString makeDate(const QDate & d);
    void attivaForeignKey(QSqlDatabase & db);
    void creaTabelle(QSqlDatabase & db);
    void salvaIntervallo(const Intervallo & i, int codwindget, QSqlDatabase & db);
    void salvaIntervalli(const QList<Intervallo> & i, int codwindget, QSqlDatabase & db);
    QList<Intervallo> prendiIntervalli(int codwidget, QSqlDatabase & db);
    void cancellaIntervalli(int codwidget, QSqlDatabase & db);
    int salvaWidget(int codups, const QString & nome, bool visibile, const QColor & colore, QSqlDatabase & db);
    void cancellaWidget(int codwidget, QSqlDatabase & db);
    void cancellaTuttiWidgetUps(int codups, QSqlDatabase & db);
    WidgetInfo prendiWidget(int codups, const QString & nome, QSqlDatabase & db);
    int salvaUps(const QString & nome, const QString & host, int porta, QSqlDatabase & db);
    void aggiornaUps(const UpsInfo & u, QSqlDatabase & db);
    void cancellaUps(int codups, QSqlDatabase & db);
    QList<UpsInfo> prendiUps(QSqlDatabase & db);
    void salvaUpsStatus(int codups, const ValoriInfo & valori, QSqlDatabase & db);
    void salvaUpsStatus(int codups, const QVector<ValoriInfo> & valori, QSqlDatabase & db);
    void cancellaUpsStatusVecchi(const QDateTime & data, QSqlDatabase & db);
    void salvaUpsCostanti(int codups, const ValoriCostantiInfo & costanti, QSqlDatabase & db);
    QList<ValoriInfo> prendiIntervalloUpsStatus(int codups, const QDateTime & inizio, const QDateTime & fine, QSqlDatabase & db);
    QList<ValoriInfo> prendiIntervalloUpsStatus(int codups, const QDate & datainizio, const QTime & orainizio, const QDate & datafine, const QTime & orafine, QSqlDatabase & db);
    void salvaParametro(const QString & parametro, bool valore, QSqlDatabase & db);
    void salvaParametro(const QString & parametro, int valore, QSqlDatabase & db);
    void salvaParametro(const QString & parametro, double valore, QSqlDatabase & db);
    void salvaParametro(const QString & parametro, QString valore, QSqlDatabase & db);
    QVariant prendiParametro(QString parametro, QSqlDatabase & db);
    QDateTime massimaData(int codups, QSqlDatabase & db);
    QDateTime minimaData(int codups, QSqlDatabase & db);
};

#endif // SQL_H
