#include "storico.h"

Storico::Storico(QList<Ups*> & list, QSqlDatabase * database, QWidget *parent) :
    QWidget(parent), upsList(list) {
  db = database;
  QGridLayout *griglia = new QGridLayout(this);

  QLabel *l;
  QPushButton *pb;

  l = new QLabel(tr("Data") + QChar(':'), this);
  griglia->addWidget(l, 0, 0, 1, 1);
  calendario = new QCalendarWidget(this);
  griglia->addWidget(calendario, 1, 0, 7, 1);
  connect(calendario, SIGNAL(selectionChanged()), SLOT(calcola()));

  l = new QLabel(tr("Ora") + QChar(':'), this);
  griglia->addWidget(l, 0, 1, 1, 1);
  orario = new QComboBox(this);
  griglia->addWidget(orario, 1, 1, 1, 1);
  for(int i = 0; i < 24; i++)
    orario->addItem(QString::number(i) + QString(" ~ ") + QString::number(i + 1), i);
  orario->setCurrentIndex(QTime::currentTime().hour());
  connect(orario, SIGNAL(currentIndexChanged(int)), SLOT(calcola()));

  l = new QLabel(tr("Ups") + QChar(':'), this);
  griglia->addWidget(l, 2, 1, 1, 2);
  ups = new QComboBox(this);
  griglia->addWidget(ups, 3, 1, 1, 2);
  connect(ups, SIGNAL(currentIndexChanged(int)), SLOT(calcolaUps(int)));
  l = new QLabel(tr("Valore ups") + QChar(':'), this);
  griglia->addWidget(l, 4, 1, 1, 2);
  valoriups = new QComboBox(this);
  griglia->addWidget(valoriups, 5, 1, 1, 2);
  valoriups->addItem(global::CaricoUPS, global::LOADPCT);
  valoriups->addItem(global::TensioneLinea, global::LINEV);
  valoriups->addItem(global::TensioneUscita, global::OUTPUTV);
  valoriups->addItem(global::FrequenzaLinea, global::LINEFREQ);
  valoriups->addItem(global::TemperaturaUPS, global::ITEMP);
  valoriups->addItem(global::CaricaBatteria, global::BCHARGE);
  valoriups->addItem(global::TensioneBatteria, global::BATTV);
  valoriups->addItem(global::TempoRimasto, global::TIMELEFT);
  valoriups->addItem(global::PotenzaErogata, global::Potenza);
  connect(valoriups, SIGNAL(currentIndexChanged(int)), SLOT(cambiagrafici(int)));

  pb = new QPushButton(tr("- 1 ora"), this);
  griglia->addWidget(pb, 6, 1, 1, 1);
  connect(pb, SIGNAL(clicked()), SLOT(oraprima()));
  pb = new QPushButton(tr("+ 1 ora"), this);
  griglia->addWidget(pb, 6, 2, 1, 1);
  connect(pb, SIGNAL(clicked()), SLOT(oradopo()));

  pb = new QPushButton(tr("Salva e aggiorna"), this);
  griglia->addWidget(pb, 7, 1, 1, 2);
  connect(pb, SIGNAL(clicked()), SLOT(salvaEaggiorna()));

  QScrollArea *scroll = new QScrollArea(this);
  griglia->addWidget(scroll, 8, 0, 1, 3);

  QWidget *w = new QWidget(scroll);
  QVBoxLayout *vlayout = new QVBoxLayout(scroll);
  for(int i = 0; i < 6; i++) {
    data_grafici[i] = new QLabel(w);
    vlayout->addWidget(data_grafici[i]);
    grafici[i] = new WidgetGrafico(w);
    vlayout->addWidget(grafici[i]);
    grafici[i]->setIntervallo(1000);
  }
  w->setLayout(vlayout);
  scroll->setWidgetResizable(true);
  scroll->setWidget(w);

  setLayout(griglia);
}

void Storico::salvaEaggiorna() {
  if(ups->currentIndex() >= 0) {
    upsList[ups->currentIndex()]->salvaValoriInMemoria();
    calcola();
    calendario->setMaximumDate(SQL::massimaData(upsList[ups->currentIndex()]->getInfo().codups, *db).date());
  }
}

void Storico::oraprima() {
  int attuale = orario->currentIndex();
  attuale--;
  if(attuale <= 0) {
    QDate data = calendario->selectedDate();
    data = data.addDays(-1);
    disconnect(calendario, SIGNAL(selectionChanged()), this, SLOT(calcola()));
    calendario->setSelectedDate(data);
    connect(calendario, SIGNAL(selectionChanged()), SLOT(calcola()));
    orario->setCurrentIndex(orario->count() - 1);
  } else
    orario->setCurrentIndex(attuale);
}

void Storico::oradopo() {
  int attuale = orario->currentIndex();
  attuale++;
  if(attuale >= orario->count()) {
    QDate data = calendario->selectedDate();
    data = data.addDays(1);
    disconnect(calendario, SIGNAL(selectionChanged()), this, SLOT(calcola()));
    calendario->setSelectedDate(data);
    connect(calendario, SIGNAL(selectionChanged()), SLOT(calcola()));
    orario->setCurrentIndex(0);
  } else
    orario->setCurrentIndex(attuale);
}

void Storico::aggiornaListaUps() {
  ups->clear();
  for(int i = 0; i < upsList.size(); i++)
    ups->addItem(upsList[i]->getInfo().nome + QString(" - ") + upsList[i]->getInfo().host, upsList[i]->getInfo().codups);
  cambiagrafici(valoriups->currentIndex());
  calcola();
}

void Storico::calcolaUps(int index) {
  if(index >= 0) {
    int codups = ups->itemData(index).toInt();
    QDate data = SQL::massimaData(codups, *db).date();
    calendario->setMaximumDate(data);
    calendario->setMinimumDate(SQL::minimaData(codups, *db).date());
    calendario->setSelectedDate(data);
    calcola();
  }
}

void Storico::cambiagrafici(int index) {
  if(ups->currentIndex() >= 0) {
    impostaColoreGrafici(index);
    calcola();
  }
}

void Storico::impostaColoreGrafici(int index) {
  global::ValoriUps v = static_cast<global::ValoriUps>(valoriups->itemData(index).toInt());
  copiaColoreGrafici(upsList[ups->currentIndex()], v);
}

void Storico::copiaColoreGrafici(Ups *u, global::ValoriUps v) {
  WidgetGrafico *grafico = dynamic_cast<WidgetGrafico*>(u->getWidgetGrafici()->getWidget(v));
  const global::ValoreSingolo &code = global::codifica(v);
  for(int i = 0; i < 6; i++) {
    grafici[i]->clear();
    grafici[i]->cancellaIntervalli();
    grafici[i]->aggiungiIntervalli(grafico->getIntervalli());
    grafici[i]->setColoreValori(grafico->getColoreValori());
    grafici[i]->setTesto(code.udmTxt, code.udm);
  }
}

void Storico::calcola() {
  if(upsList.count() > 0) {
    QDateTime dataprima;
    dataprima.setDate(calendario->selectedDate());
    dataprima.setTime(QTime(orario->currentIndex(), 0, 0));
    QList<SQL::ValoriInfo> valori;
    for(int i = 0; i < 6; i++) {
      QDateTime datadopo = dataprima.addSecs(600);
      data_grafici[i]->setText(QString("<b>") +
                               dataprima.toString(QString("ddd d MMM yyyy h:m:s")).toUpper() +
                               QString(" &rarr; ") +
                               datadopo.toString(QString("h:m:s")) +
                               QChar(' ') +
                               global::codifica(static_cast<global::ValoriUps>(valoriups->itemData(valoriups->currentIndex()).toInt())).udmTxt +
                               QString(":</b>"));
      valori.clear();
      valori = SQL::prendiIntervalloUpsStatus(upsList[ups->currentIndex()]->getInfo().codups, dataprima, datadopo, *db);
      grafici[i]->clear();
      for(int j = 0; j < valori.size(); j++)
        switch(static_cast<global::ValoriUps>(valoriups->itemData(valoriups->currentIndex()).toInt())) {
          case global::LOADPCT:
            grafici[i]->pushValore(valori[j].LOADPCT, valori[j].DATE, false, false);
          break;
          case global::LINEV:
            grafici[i]->pushValore(valori[j].LINEV, valori[j].DATE, false, false);
          break;
          case global::OUTPUTV:
            grafici[i]->pushValore(valori[j].OUTPUTV, valori[j].DATE, false, false);
          break;
          case global::LINEFREQ:
            grafici[i]->pushValore(valori[j].LINEFREQ, valori[j].DATE, false, false);
          break;
          case global::ITEMP:
            grafici[i]->pushValore(valori[j].ITEMP, valori[j].DATE, false, false);
          break;
          case global::BCHARGE:
            grafici[i]->pushValore(valori[j].BCHARGE, valori[j].DATE, false, false);
          break;
          case global::BATTV:
            grafici[i]->pushValore(valori[j].BATTV, valori[j].DATE, false, false);
          break;
          case global::TIMELEFT:
            grafici[i]->pushValore(valori[j].TIMELEFT, valori[j].DATE, false, false);
          break;
          case global::Potenza:
            grafici[i]->pushValore(upsList[ups->currentIndex()]->getCostanti().NOMPOWER * valori[j].LOADPCT / 100, valori[j].DATE, false, false);
          break;
          default:
            break;
        }
      grafici[i]->pulisciValori();
      grafici[i]->ridisegna();
      dataprima = datadopo;
    }
  }
}
