#include "ups.h"

Ups::Ups(const SQL::UpsInfo & i, QObject *parent) :
    QObject(parent), ups(i) {
  tabwidgetinterno = 0;
  widgetbarre = 0;
  miniUPS = 0;
  db = 0;
  primo = true;
  net.setIndirizzo(i.host, i.porta);
  connect(&net, SIGNAL(newConstants()), SLOT(getNewConstant()));
  connect(&net, SIGNAL(newEvents()), SLOT(getNewEvents()));
  connect(&net, SIGNAL(newValues()), SLOT(getNewValue()));
}

Ups::~Ups() {
  stop();
  waitStop();
  salvaValoriInMemoria();
}

void Ups::salvaValoriInMemoria() {
  int ultimo = valori.size() - 1;
  if(valori.size() >= 2 && valori[ultimo].isEquivalent(valori[ultimo - 1]))
    valori.pop_back();
  if(db)
    SQL::salvaUpsStatus(ups.codups, valori, *db);
  valori.clear();
  primo = false;
}

void Ups::getNewValue() {
  SQL::ValoriInfo v = net.getNewValue();
  int ultimo = valori.size() - 1;
  if(ultimo < 0) {
    if(primo || v.DATE != ultimaDataOra) {
      valori.push_back(v);
      ultimaDataOra = v.DATE;
      emit nuovoValore(v);
      emit nuovoGrezzo(net.getNewAPCStatus());
    }
  } else if(valori[ultimo] != v) {
    if(valori.size() >= 2 && valori[ultimo].isEquivalent(v) && valori[ultimo - 1].isEquivalent(v))
      valori[ultimo] = v;
    else
      valori.push_back(v);
    ultimaDataOra = v.DATE;
    emit nuovoValore(v);
    emit nuovoGrezzo(net.getNewAPCStatus());
    if(valori.size() > ups.maxvalori) {
      salvaValoriInMemoria();
    }
  }
}

void Ups::getNewConstant() {
  SQL::ValoriCostantiInfo c = net.getNewConstant();
  if(costanti != c) {
    costanti = c;
    SQL::salvaUpsCostanti(ups.codups, c, *db);
    emit nuoveCostanti(c);
  }
}

void Ups::getNewEvents() {
  emit nuoviEventi(net.getNewAPCEvents());
}

SQL::ValoriCostantiInfo Ups::getCostanti() const {
  return costanti;
}

void Ups::setDB(QSqlDatabase *database) {
  db = database;
}

void Ups::setWidgetBarre(ListaWidgetAdIntervalli *w) {
  widgetbarre = w;
}

void Ups::setWidgetGrafici(ListaWidgetAdIntervalli *w) {
  widgetgrafici = w;
}

void Ups::setMiniUpsWidget(QWidget *mini) {
  miniUPS = mini;
}

void Ups::setTabWidget(QTabWidget *tab) {
  tabwidgetinterno = tab;
}

void Ups::start() {
  if(!net.isRunning())
    net.start();
}

void Ups::stop() {
  if(net.isRunning())
    net.termina();
}

void Ups::waitStop() {
  net.wait();
}

SQL::UpsInfo Ups::getInfo() const {
  return ups;
}

QTabWidget * Ups::getTabWidget() {
  return tabwidgetinterno;
}

ListaWidgetAdIntervalli * Ups::getWidgetBarre() {
  return widgetbarre;
}

ListaWidgetAdIntervalli * Ups::getWidgetGrafici() {
  return widgetgrafici;
}

QWidget * Ups::getMiniUpsWidget() {
  return miniUPS;
}

void Ups::setInfo(const SQL::UpsInfo & i) {
  if(i.host != ups.host || i.porta != ups.porta) {
    net.setIndirizzo(i.host, i.porta);
  }
  ups = i;
  SQL::aggiornaUps(ups, *db);
}

void Ups::askEvents() {
  net.askEvents();
}
