#include "widgetbarra.h"

WidgetBarra::WidgetBarra(QWidget *parent) : WidgetAdIntervalli(parent) {
  barremostrate = 1;
  setMinimumHeight(altezzaMinima);
  setMinimumWidth(100);
}

const int WidgetBarra::altezzaMinima(30);

void WidgetBarra::setNumeroBarre(int n) {
  if(n > 0 && n < 20) {
    barremostrate = n;
    setMinimumHeight(altezzaMinima + barremostrate * 3);
    update();
  }
}

void WidgetBarra::pushValore(double val) {
  QList<double> old = valori;
  valori.push_front(val);
  while(valori.size() > barremostrate)
    valori.pop_back();
  if(valori != old)
    update();
}

void WidgetBarra::paintEvent(QPaintEvent *event) {
  event->ignore();
  QPainter paint;
  paint.begin(this);

  int h = height() - 2;
  int w = width() - 2;
  int x1 = 1;
  int y1 = 16;
  int x2 = w - x1;
  int y2 = h - y1;
  int c_h, c_s, c_v, c_a;
  QColor colore;

  paint.setPen(coloretesto);
  if(valori.size() < 1)
    paint.drawText(QRect(1, 1, w, y1), empty);
  else {
    paint.drawText(QRect(1, 1, w, y1), label + QString(" : ") + QString::number(valori[0]) + QChar(' ') + udm);
    if(bordo)
      paint.setPen(colorebordo);
    else
      paint.setPen(QColor(0, 0, 0, 0));
    paint.setBrush(Qt::white);
    paint.drawRect(x1, y1, x2, y2);
    int intervallini = (float)y2 / (float)(valori.size() + 3);
    double min = trovaMin();
    double max = trovaMax();
    double distanza = qAbs(max - min);

    // SFONDI INTERVALLI
    for(int i = 0; i < intervalli.size(); i++) {
      int sotto = intervalli[i].getSotto();
      int sopra = intervalli[i].getSopra();
      if(sotto < min || sotto > max)
        sotto = min;
      if(sopra < min || sopra > max)
        sopra = max;
      if(intervalli[i].getSfumatura()) {
        QLinearGradient gradiente(QPoint(x1, y1), QPoint(x1, y1 + y2));
        gradiente.setColorAt(0, Qt::white);
        gradiente.setColorAt(1, intervalli[i].getColore());
        paint.setBrush(QBrush(gradiente));
      } else
        paint.setBrush(intervalli[i].getColore());
      paint.drawRect(qFloor(x1 + x2 * qAbs(sotto - min) / distanza), y1, qCeil(x2 * qAbs(sopra - sotto) / distanza), y2);
    }

    // BARRE
    colorevalori.getHsv(&c_h, &c_s, &c_v, &c_a);
    int saturazione = c_s / (valori.size());
    for(int i = valori.size() - 1, j = 1; i >= 0; i--, j++) {
      int lunghezza = 0;
      if(qFloor(valori[i]) >= max)
        lunghezza = x2;
      else if(valori[i] > min)
        lunghezza = x2 * qAbs(valori[i] - min) / distanza + 1;
      if(sfumatura) {
        colore.setHsv(c_h, saturazione * j, c_v, 255);
        QLinearGradient gradiente(QPoint(0, 0), QPoint(lunghezza, 0));
        gradiente.setColorAt(0, Qt::white);
        gradiente.setColorAt(1, colore);
        paint.setBrush(QBrush(gradiente));
      } else
        paint.setBrush(colorevalori);
      if(i > 0)
        paint.drawRect(x1, y1 + intervallini * j, lunghezza, 2 * intervallini);
      else
        paint.drawRect(x1, y1 + intervallini * j, lunghezza, y2 - intervallini * (j + 1));
    }
  }

  paint.end();
}

