#include "widgetgrafico.h"

WidgetGrafico::WidgetGrafico(QWidget *parent) :
    WidgetAdIntervalli(parent) {
  intervallo = 120;
  coloregriglia = Qt::gray;
  coloreevidenziato = Qt::yellow;
  barreorizzontali = 5;
  xmouse = 0;
  ymouse = 0;
  setMouseTracking(true);
  setMinimumSize(300, 150);
}

void WidgetGrafico::mouseMoveEvent(QMouseEvent *evento) {
  xmouse = evento->x();
  ymouse = evento->y();
  ridisegna();
}

void WidgetGrafico::setIntervallo(int i) {
  intervallo = i;
  ridisegna();
}

void WidgetGrafico::pushValore(double valore, const QDateTime & orario, bool repainta, bool pulisci) {
  Punti p;
  p.valore = valore;
  p.orario = orario;
  if(valori.size() <= 0) {
    valori.push_back(p);
    if(repainta)
      ridisegna();
  } else {
    if(valori[valori.size() - 1] == p)
      valori[valori.size() -1] = p;
    else {
      valori.push_back(p);
      qSort(valori);
      if(pulisci)
        pulisciValori();
      if(repainta)
        ridisegna();
    }
  }
}

void WidgetGrafico::pulisciValori() {
  if(valori.size() > 0) {
    QDateTime &ultimo = valori[valori.size() - 1].orario;
    QVector<Punti>::Iterator it = valori.begin();
    bool fine = false;
    while(!fine) {
      if(it->orario.secsTo(ultimo) >= intervallo)
        it = valori.erase(it);
      else
        fine = true;
    }
  }
}

void WidgetGrafico::ridisegna() {
  update();
}

bool WidgetGrafico::Punti::operator==(const WidgetGrafico::Punti & p) const {
  return orario == p.orario;
}

bool WidgetGrafico::Punti::operator<(const WidgetGrafico::Punti & p) const {
  return orario < p.orario;
}

void WidgetGrafico::setColoreGriglia(const QColor & c) {
  coloregriglia = c;
  ridisegna();
}

void WidgetGrafico::setColoreEvidenziato(const QColor & c) {
  coloreevidenziato = c;
  ridisegna();
}

void WidgetGrafico::setOrizzontali(int o) {
  barreorizzontali = o;
}

void WidgetGrafico::clear(bool ridisegna) {
  valori.clear();
  if(ridisegna)
    repaint();
}

void WidgetGrafico::leaveEvent(QEvent *event) {
  event->ignore();
  xmouse = 0;
  ymouse = 0;
  ridisegna();
}

void WidgetGrafico::paintEvent(QPaintEvent *event) {
  event->ignore();
  QPainter paint;
  paint.begin(this);

  int h = height() - 2;
  int w = width() - 2;
  int x1 = 1;
  int y1 = 17;
  int x2 = w - x1 - 37;
  int y2 = h - y1 - 17;
  int c_h, c_s, c_v, c_a;
  QColor colore;

  paint.setPen(coloretesto);
  if(valori.size() < 1)
    paint.drawText(QRect(1, 1, w, y1), empty);
  else if(intervalli.size() < 1)
    paint.drawText(QRect(1, 1, w, y1), tr("Nessun intervallo"));
  else {
    paint.drawText(QRect(1, 1, w, y1), label + QString(" (") + udm + QChar(')'));
    if(bordo)
      paint.setPen(colorebordo);
    else
      paint.setPen(QColor(0, 0, 0, 0));
    paint.setBrush(Qt::white);
    paint.drawRect(x1 - 1, y1 - 1, x2 + 1, y2 + 1);
    double min = trovaMin();
    double max = trovaMax();
    double distanza = qAbs(max - min);

    // SFONDI INTERVALLI
    paint.setPen(QColor(0, 0, 0, 0));
    for(int i = 0; i < intervalli.size(); i++) {
      int sotto = intervalli[i].getSotto();
      int sopra = intervalli[i].getSopra();
      if(sotto < min || sotto > max)
        sotto = min;
      if(sopra < min || sopra > max)
        sopra = max;
      if(intervalli[i].getSfumatura()) {
        QLinearGradient gradiente(QPoint(x1, y1), QPoint(x1, y1 + y2));
        intervalli[i].getColore().getHsv(&c_h, &c_s, &c_v, &c_a);
        c_s = 25;
        colore.setHsv(c_h, c_s, c_v, c_a);
        gradiente.setColorAt(0, colore);
        c_s = 100;
        colore.setHsv(c_h, c_s, c_v, c_a);
        gradiente.setColorAt(1, colore);
        paint.setBrush(QBrush(gradiente));
      } else
        paint.setBrush(intervalli[i].getColore());
      paint.drawRect(x1, qCeil(y1 + y2 - y2 * qAbs(sotto - min) / distanza), x2, qFloor(-y2 * qAbs(sopra - sotto) / distanza));
    }

    // UDM
    double steporizzontali = (double)y2 / (barreorizzontali + 1);
    paint.setPen(coloregriglia);
    for(int i = 0; i < barreorizzontali; i++)
      paint.drawLine(x1, y1 + steporizzontali * (i + 1), x1 + x2 - 1, y1 + steporizzontali * (i + 1));
    paint.setPen(coloretesto);
    for(int i = 0; i < barreorizzontali + 2; i++)
      paint.drawText(x1 + x2 + 2, y1 + steporizzontali * i + 5, QString::number(max - (max - min) / (barreorizzontali + 1) * i, 'f', 1));

    // VALORI (e linee verticali)
    Punti *p = 0;
    int intervallo = valori[0].orario.secsTo(valori[valori.size() - 1].orario);
    double step = (double)x2 / (intervallo > 0 ? intervallo : 1);
    QPointF old;
    QPoint punto;
    punto.setX(x1 + 1); // +1 solo x antialiasing
    int altezza;
    if(valori[0].valore >= max)
      altezza = y1 + 1;  // +1 solo x antialiasing
    else if(valori[0].valore <= min)
      altezza = y1 + y2 - 1;
    else
      altezza = y1 + y2 - y2 * qAbs(valori[0].valore - min) / distanza - 1;
    punto.setY(altezza);
    old.setX(punto.x());
    old.setY(punto.y());
    QPainterPath path(punto);
    if(ymouse > y1 && ymouse < y1 + y2 && xmouse >= x1 && xmouse <= x1 + 1) {
      p = &valori[0];
      paint.setPen(coloreevidenziato);
      paint.drawLine(x1, y1, x1, y1 + y2 - 1);
      paint.drawLine(x1 + 1, y1, x1 + 1, y1 + y2 - 1);
    }
    for(int i = 1; i < valori.size(); i++) {
      QPoint puntosotto, puntosopra;
      double newx = valori[0].orario.secsTo(valori[i].orario) * step + x1 - 1;
      punto.setX(qRound(newx));
      if(valori[i].valore >= max)
        altezza = y1 + 1;
      else if(valori[i].valore <= min)
        altezza = y1 + y2 - 1;
      else
        altezza = y1 + y2 - y2 * qAbs(valori[i].valore - min) / distanza - 1;
      punto.setY(altezza);
      puntosotto.setX((old.x() + newx) / 2);
      puntosotto.setY(old.y());
      puntosopra.setX(puntosotto.x());
      puntosopra.setY(punto.y());
      old.setX(newx);
      old.setY(punto.y());
      path.cubicTo(puntosotto, puntosopra, punto);
      if(i != valori.size() - 1) {
        paint.setPen(coloregriglia);
        paint.drawLine(punto.x(), y1, punto.x(), y1 + y2 - 1);
      }
      if(ymouse > y1 && ymouse < y1 + y2 && xmouse >= punto.x() - 1 && xmouse <= punto.x() + 1) {
        paint.setPen(coloreevidenziato);
        p = &valori[i];
        if(xmouse > x1)
          paint.drawLine(punto.x() - 1, y1, punto.x() - 1, y1 + y2 - 1);
        if(xmouse < x1 + x2 - 2)
          paint.drawLine(punto.x() + 1, y1, punto.x() + 1, y1 + y2 - 1);
        if(xmouse >= x1 + x2 - 2 && xmouse <= x1 + x2)
          paint.drawLine(punto.x(), y1, punto.x(), y1 + y2 - 1);
      }
    }
    QPen penna(colorevalori);
    penna.setWidth(2);
    paint.setPen(penna);
    paint.setBrush(QColor(0, 0, 0, 0));
    paint.setRenderHint(QPainter::Antialiasing, true);
    paint.drawPath(path);
    paint.setRenderHint(QPainter::Antialiasing, false);

    // TESTO valore selezionato
    if(p) {
      paint.setPen(coloretesto);
      paint.drawText(x1, y1 + y2 + 13, tr("Data") + QString(": ") +
                     p->orario.toString(QString("d/M/yyyy")) + QString("   ") +
                     tr("Ora") + QString(": ") +
                     p->orario.toString(QString("h:m:s")));
      paint.drawText(x1 + 220, y1 + y2 + 13, tr("Valore") + QString(": ") +
                     QString::number(p->valore) + QChar(' ') + udm);
    }
  }

  paint.end();
}
